﻿/* Clockwise Matrix
 * Gordon Lee
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Test
{
    public partial class Form1 : Form
    {
        // Member variables
        private Matrix m_matrix = new Matrix();

        public Form1()
        {
            InitializeComponent();
        }

        // Matrix Output Clockwise
        private void MatrixOutput_Button_Click(object sender, EventArgs e)
        {
            // Matrix Input
            string _matrixInputString = MatrixInput_TextBox.Text;
            try
            {
                m_matrix.ParseMatrixInput(_matrixInputString);
            }
            catch (Matrix.BadMatrixException)
            {
                return;
            }

            // Rows and Columns
            Rows_TextBox.Text = m_matrix.GetNumRows().ToString();
            Columns_TextBox.Text = m_matrix.GetNumColumns().ToString();

            // Matrix Form
            string _matrixFormString = "";
            for (int i = 0; i < m_matrix.GetNumRows(); ++i)
            {
                for (int j = 0; j < m_matrix.GetNumColumns(); ++j)
                {
                    _matrixFormString += m_matrix.GetValue(i, j).ToString() + "\t";
                }
                _matrixFormString = _matrixFormString.Remove(_matrixFormString.Length - 1);
                _matrixFormString += Environment.NewLine;
            }
            _matrixFormString = _matrixFormString.Remove(_matrixFormString.Length - 2);
            MatrixForm_TextBox.Text = _matrixFormString;

            // Output
            List<double> _output = m_matrix.OutputClockwise();
            string _matrixOutputString = "";
            foreach (double _value in _output)
            {
                _matrixOutputString += _value + " ";
            }
            _matrixOutputString = _matrixOutputString.Remove(_matrixOutputString.Length - 1);
            MatrixOutput_TextBox.Text = _matrixOutputString;
        }

        // Form1 Size Changed
        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            Point _p = new Point();

            // Matrix Input Label
            _p.X = Width / 2 - MatrixInput_Label.Width / 2;
            _p.Y = MatrixInput_Label.Location.Y;
            MatrixInput_Label.Location = _p;

            // Matrix Form Label
            _p.X = Width / 2 - MatrixForm_Label.Width / 2;
            _p.Y = MatrixForm_Label.Location.Y;
            MatrixForm_Label.Location = _p;

            // Matrix Output Label
            _p.X = Width / 2 - MatrixOutput_Label.Width / 2;
            _p.Y = MatrixOutput_TextBox.Location.Y - 20;
            MatrixOutput_Label.Location = _p;

            // Matrix Output Clockwise Button
            _p.X = MatrixInput_TextBox.Location.X + MatrixInput_TextBox.Width + 28;
            _p.Y = MatrixOutput_Button.Location.Y;
            MatrixOutput_Button.Location = _p;

            // Rows Label
            _p.Y = Rows_Label.Location.Y;
            Rows_Label.Location = _p;

            // Columns Label
            _p.Y = Columns_Label.Location.Y;
            Columns_Label.Location = _p;

            // Rows TextBox
        }

        // Menu - New Matrix
        private void newMatrixToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MatrixInput_TextBox.Clear();
            MatrixForm_TextBox.Clear();
            MatrixOutput_TextBox.Clear();
            Rows_TextBox.Clear();
            Columns_TextBox.Clear();
        }

        // Menu - Save Matrix
        private void saveMatrixToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog _saveFileDialog1 = new SaveFileDialog();
            _saveFileDialog1.Title = "Save Matrix";
            _saveFileDialog1.InitialDirectory = Application.StartupPath;
            _saveFileDialog1.Filter = "Matrix Files (*.mtx)|*.mtx|All Files (*.*)|*.*";
            _saveFileDialog1.FilterIndex = 0;
            _saveFileDialog1.RestoreDirectory = true;
            _saveFileDialog1.OverwritePrompt = true;

            if (_saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                string _path = _saveFileDialog1.FileName;
                FileStream _file = new FileStream(_path, FileMode.OpenOrCreate, FileAccess.Write);

                StreamWriter _sw = new StreamWriter(_file);
                try
                {
                    _sw.Write(MatrixInput_TextBox.Text);
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.ToString(), "Error");
                }
                finally
                {
                    _sw.Close();
                    _file.Close();
                }
            }
        }

        // Menu - Load Matrix
        private void loadMatrixToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog _openFileDialog1 = new OpenFileDialog();
            _openFileDialog1.Title = "Open Matrix";
            _openFileDialog1.InitialDirectory = Application.StartupPath;
            _openFileDialog1.Filter = "Matrix Files (*.mtx)|*.mtx|All Files (*.*)|*.*";
            _openFileDialog1.FilterIndex = 0;
            _openFileDialog1.RestoreDirectory = true;

            if (_openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                // Clear everything
                MatrixInput_TextBox.Clear();
                MatrixForm_TextBox.Clear();
                MatrixOutput_TextBox.Clear();
                Rows_TextBox.Clear();
                Columns_TextBox.Clear();

                string _path = _openFileDialog1.FileName;
                FileStream _file = new FileStream(_path, FileMode.Open, FileAccess.Read);

                StreamReader _sr = new StreamReader(_file);
                try
                {
                    MatrixInput_TextBox.Text = _sr.ReadToEnd();
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.ToString(), "Error");
                }
                finally
                {
                    _sr.Close();
                    _file.Close();
                }
            }
        }

        // Menu - Exit
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void howToToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Input Matrix using MATLAB matrix format.\r\n" +
                "ex: [1, 2, 3; 4, 5, 6; 7, 8, 9]\r\n" +
                "Must begin with [ and end with ]\r\n" +
                "Columns within rows are split using commas (,)\r\n" +
                "Rows are split using semi-colons (;)\r\n" +
                "Extra spaces and new lines are available for use.\r\n\r\n" +
                "Click the Output Clockwise Matrix button to the right of the Matrix Input field to produce the result.\r\n" +
                "The inputted matrix will appear in normal Matrix Form, the clockwise matrix will appear as a list in the Matrix Output.\r\n\r\n" +
                "Clear everything in File -> New Matrix\r\n" +
                "Save Input Matrix in File -> Save Matrix\r\n" +
                "Load Saved Matrix in File -> Load Matrix\r\n" +
                "Matrices are saved as a .mtx file",
                "How To");
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Clockwise Matrix\r\n" +
                "By: Gordon Lee\r\n" +
                "Written in C# in Microsoft Visual Studio 2008\r\n" +
                "Outputs a matrix in a clockwise manner without repeats",
                "About");
        }
    }
}
